const fs = require('fs-extra');
const path = require('path');
const { Storage, getUploadDirFileList } = require('../lib/utils/storage');
const { AsyncTaskParallelController } = require('@cloudbase/manager-node/lib/utils/parallel');

const config = {};

try {
  const extra_config = require('./extra_config');
  Object.assign(config, extra_config);
} catch (e) {}

const instance = new Storage({
  endpoint: 'http://oss-csp1.rgw.infra.tce.io',
  region: 'us-east-1',
  credential: config.S3_Credential,
  cos: false,
});

describe('s3 upload download 测试', () => {
  it.skip('list bucket', async () => {
    const res = await new Promise((resolve, reject) => {
      instance.listBuckets((err, data) => {
        if (err) {
          reject(err);
          return;
        }
        resolve(data);
      });
    });

    expect(!res.Buckets.length).toBe(true);
  });

  it('put obj', async () => {
    const res = await instance.putObject({
      ACL: 'public-read',
      Bucket: config.S3_Bucket,
      Body: fs.readFileSync(path.resolve(__dirname, './dist.zip')),
      Key: 'code.zip',
    });
    expect(res).toBe(null);
  });

  it.skip('get obj signed url', async () => {
    const res = await instance.getObjectSignedUrl({
      Bucket: config.S3_Bucket,
      Key: 'code.zip',
      Expires: 3600,
    });
    console.log(res);
    expect(!res.Url).toBe(false);
  });

  it.skip('mutil put object', async () => {
    const deployContent = [
      {
        src: path.resolve(__dirname, '../lib'),
        cloudPath: 'multi_test',
      },
    ];
    const fileList = deployContent.reduce((list, meta) => {
      list.push(...getUploadDirFileList(meta.src, meta.cloudPath));
      return list;
    }, []);

    const asyncTaskController = new AsyncTaskParallelController(50, 50);
    asyncTaskController.loadTasks(
      fileList.map((meta) => {
        return async () => {
          try {
            const param = {
              Bucket: config.S3_Bucket,
              ACL: 'public-read',
              Body: fs.createReadStream(meta.src),
              Key: meta.cloudPath,
            };
            try {
              await instance.putObject(param);
            } catch (e) {
              await new Promise((resolve, reject) => {
                setTimeout(() => {
                  resolve(null);
                }, 1000);
              });
              await instance.putObject(param);
            }
            return { code: 0 };
          } catch (e) {
            if (!e.code) {
              e.code = -1;
            }
            throw e;
          }
        };
      }),
    );

    const resList = await asyncTaskController.run();
    expect(resList.filter((item) => item.code).length).toBe(0);
  });
});
