"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = void 0;
/**
 * Tencent is pleased to support the open source community by making CloudBaseFramework - 云原生一体化部署工具 available.
 *
 * Copyright (C) 2020 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * Please refer to license text included with this package for license details.
 */
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const url_1 = __importDefault(require("url"));
const child_process_1 = require("child_process");
const util_1 = require("util");
const framework_core_1 = require("@cloudbase/framework-core");
const CI = __importStar(require("miniprogram-ci"));
const SUPPORT_DEPLOY_MODE = ['upload', 'preview'];
const MP_CONFIG_FILENAME = 'project.config.json';
const NOT_NPM_ERROR = '__NO_NODE_MODULES__ NPM packages not found';
class MiniProgramsPlugin extends framework_core_1.Plugin {
    constructor(name, api, inputs) {
        super(name, api, inputs);
        this.name = name;
        this.api = api;
        this.inputs = inputs;
        const DEFAULT_INPUTS = {
            localPath: './',
            deployMode: 'preview',
            ignores: ['node_modules/**/*'],
        };
        this.resolvedInputs = resolveInputs(this.inputs, DEFAULT_INPUTS);
        this.buildOutput = {};
    }
    /**
     * 初始化
     */
    async init() {
        this.api.logger.debug('MiniProgramPlugin: init', this.resolvedInputs);
    }
    _initCI() {
        const { projectPath } = this.api;
        const { appid, privateKeyPath, privateKey, localPath, deployMode, } = this.resolvedInputs;
        if (!appid) {
            throw new Error('小程序 appid 不能为空，请在 cloudbaserc.json 中指明 appid. 小程序 appid 一般可以在 project.config.json 中找到');
        }
        // 存在私钥则直接用私钥
        // 不存在私钥就尝试从私有路径中获取
        if (!privateKey) {
            if (!privateKeyPath ||
                !fs_1.default.existsSync(path_1.default.resolve(projectPath, privateKeyPath))) {
                throw new Error('找不到小程序的部署私钥，请在 cloudbaserc.json 指明私钥文件路径 privateKeyPath. 小程序的部署私钥可在微信公众平台上登录后获取');
            }
        }
        else {
            const keyPath = path_1.default.resolve(projectPath, './private.key');
            const key = Buffer.from(privateKey, 'base64');
            fs_1.default.writeFileSync(keyPath, key.toString());
            this.resolvedInputs.privateKeyPath = keyPath;
        }
        if (deployMode && !SUPPORT_DEPLOY_MODE.includes(deployMode)) {
            throw new Error(`CloudBase Framework: 不支持的小程序部署模式 '${deployMode}'`);
        }
        if (!fs_1.default.existsSync(path_1.default.resolve(projectPath, localPath, MP_CONFIG_FILENAME))) {
            throw new Error('项目内找不到小程序配置文件 project.config.json，请在 cloudbaserc.json 中指明小程序应用的项目路径 localPath.');
        }
    }
    /**
     * 执行本地命令
     */
    async run() { }
    /**
     * 删除资源
     */
    async remove() { }
    /**
     * 生成代码
     */
    async genCode() { }
    /**
     * 构建
     */
    async build() {
        var _a;
        this.api.logger.debug('MiniProgramPlugin: build', this.resolvedInputs);
        const { build: buildCommand, install: installCommand } = ((_a = this.resolvedInputs) === null || _a === void 0 ? void 0 : _a.commands) || {};
        /**
         * 安装依赖
         */
        if (installCommand) {
            this.api.logger.info(installCommand);
            await (0, util_1.promisify)(child_process_1.exec)(installCommand);
        }
        /**
         * 构建
         */
        if (buildCommand) {
            this.api.logger.info(buildCommand);
            await (0, util_1.promisify)(child_process_1.exec)(buildCommand);
        }
        return;
    }
    /**
     * 生成SAM文件
     */
    async compile() {
        /**
         * todo: 小程序部署应该迁移到SAM平台
         */
        var _a;
        /**
         * 初始化CI
         */
        this._initCI();
        /**
         * 编译小程序NPM
         */
        await this._ciPackNpm();
        /**
         * 部署小程序
         */
        switch ((_a = this.resolvedInputs) === null || _a === void 0 ? void 0 : _a.deployMode) {
            case 'upload': {
                this.buildOutput = await this._ciUpload();
                break;
            }
            case 'preview': {
                this.buildOutput = await this._ciPreview();
                break;
            }
            default: {
                break;
            }
        }
        return {};
    }
    /**
     * 部署
     */
    async deploy() {
        this.api.logger.debug('MiniProgramPlugin: deploy', this.resolvedInputs, this.buildOutput);
        const { link, version } = this.buildOutput || {};
        if (link) {
            this.api.logger.info(`${this.api.emoji('🚀')} 小程序（预览版）部署成功，预览二维码地址：${link}`);
        }
        else if (version) {
            this.api.logger.info(`${this.api.emoji('🚀')} 小程序（体验版v${version}）上传成功，请在小程序管理后台将其设置为体验版本`);
        }
        else {
            throw new Error(`invalid buildOutput: ${this.buildOutput}`);
        }
    }
    /**
     * 小程序-上传
     */
    async _ciUpload() {
        // 需要暂时关掉 stdout, 避免 miniprogram-ci 的内容打印到控制台
        // this.api.runtime.isLocal() && this.api.console.pause();
        !process.env.CLOUDBASE_CIID && pauseConsoleOutput();
        const { version = '1.0.0', desc = 'CloudBase Framework 一键上传', setting, } = this.resolvedInputs.uploadOptions || {};
        const result = await CI.upload({
            project: this._ciProject,
            version,
            desc,
            setting,
        }).catch((err) => {
            return err;
        });
        // this.api.runtime.isLocal() && this.api.console.resume();
        !process.env.CLOUDBASE_CIID && resumeConsoleOutput();
        if (result === null || result === void 0 ? void 0 : result.subPackageInfo) {
            return {
                version,
            };
        }
        else {
            throw new Error(`小程序（体验版）部署失败 ${result}`);
        }
    }
    /**
     * 小程序-预览
     */
    async _ciPreview() {
        // 需要暂时关掉 stdout, 避免 miniprogram-ci 的内容打印到控制台
        // this.api.runtime.isLocal() && this.api.console.pause();
        !process.env.CLOUDBASE_CIID && pauseConsoleOutput();
        const { desc = 'CloudBase Framework 一键预览', setting, qrcodeOutputPath = './qrcode.jpg', pagePath = 'pages/index/index', searchQuery = '', scene = 1011, } = this.resolvedInputs.previewOptions || {};
        const result = await CI.preview({
            project: this._ciProject,
            version: '0.0.1',
            desc,
            setting,
            qrcodeFormat: 'image',
            qrcodeOutputDest: path_1.default.resolve(this.api.projectPath, qrcodeOutputPath),
            pagePath,
            searchQuery,
            scene,
        }).catch((err) => {
            return err;
        });
        // this.api.runtime.isLocal() && this.api.console.resume();
        !process.env.CLOUDBASE_CIID && resumeConsoleOutput();
        if (result === null || result === void 0 ? void 0 : result.subPackageInfo) {
            const link = this.api.genClickableLink(url_1.default.format({
                protocol: 'file:',
                host: path_1.default.resolve(this.api.projectPath, qrcodeOutputPath),
            }));
            return {
                link,
            };
        }
        else {
            throw new Error(`小程序（预览版）部署失败 ${result}`);
        }
    }
    /**
     * 小程序-编译NPM
     */
    async _ciPackNpm() {
        // 需要暂时关掉 stdout, 避免 miniprogram-ci 的内容打印到控制台
        // this.api.runtime.isLocal() && this.api.console.pause();
        !process.env.CLOUDBASE_CIID && pauseConsoleOutput();
        const result = await CI.packNpm(this._ciProject, {
            reporter: (infos) => {
                console.log(infos);
            },
        }).catch((err) => {
            return err;
        });
        // this.api.runtime.isLocal() && this.api.console.resume();
        !process.env.CLOUDBASE_CIID && resumeConsoleOutput();
        if (result instanceof Error && !result.message.startsWith(NOT_NPM_ERROR)) {
            throw new Error(`小程序 NPM 构建失败 ${result}`);
        }
    }
    get _ciProject() {
        const { projectPath } = this.api;
        const { appid, localPath, privateKeyPath, ignores } = this.resolvedInputs;
        return new CI.Project({
            appid,
            type: 'miniProgram',
            projectPath: path_1.default.resolve(projectPath, localPath),
            privateKeyPath: path_1.default.resolve(projectPath, privateKeyPath),
            ignores,
        });
    }
}
function resolveInputs(inputs, defaultInputs) {
    return Object.assign({}, defaultInputs, inputs);
}
const originalStdoutWrite = process.stdout.write.bind(process.stdout);
const originalStderrWrite = process.stderr.write.bind(process.stderr);
let previousStdoutWrite = process.stdout.write.bind(process.stdout);
let previousStderrWrite = process.stderr.write.bind(process.stderr);
// 暂停控制台输出
function pauseConsoleOutput() {
    previousStdoutWrite = process.stdout.write.bind(process.stdout);
    process.stdout.write = () => {
        return true;
    };
    previousStderrWrite = process.stderr.write.bind(process.stderr);
    process.stderr.write = () => {
        return true;
    };
}
// 恢复控制台输出
function resumeConsoleOutput(original = false) {
    if (original) {
        process.stdout.write = originalStdoutWrite;
        process.stderr.write = originalStderrWrite;
    }
    else {
        process.stdout.write = previousStdoutWrite;
        process.stderr.write = previousStderrWrite;
    }
}
exports.plugin = MiniProgramsPlugin;
