export = AbstractClient;
/**
 * @inner
 */
declare class AbstractClient {
    /**
     * Initialize the client.
     * @param {string} endpoint The service's domain name, such as [product].tcecloudapi.com.
     * @param {string} version The version of the service, such as 2017-03-12.
     * @param {Credential} credential Credentials.
     * @param {string} region Region of the service.
     * @param {ClientProfile} profile Profile settings.
     */
    constructor(endpoint: string, version: string, credential: Credential, region: string, profile: ClientProfile);
    path: string;
    /**
     * Credentials.
     * @type {Credential || null}
     */
    credential: Credential;
    /**
     * Region of the service.
     * @type {string || null}
     */
    region: string;
    sdkVersion: string;
    apiVersion: string;
    endpoint: string;
    /**
     * Optional profile settings.
     * @type {ClientProfile}
     */
    profile: ClientProfile;
    /**
     * @inner
     */
    getEndpoint(): string;
    /**
     * @inner
     */
    succRequest(resp: any, cb: any, data: any): void;
    /**
     * @inner
     */
    failRequest(err: any, cb: any): void;
    /**
     * @inner
     */
    request(action: any, req: any, resp: any, cb: any): void;
    /**
     * @inner
     */
    doRequest(action: any, req: any): Promise<any>;
    /**
     * @inner
     */
    mergeData(data: any, prefix?: string): {};
    /**
     * @inner
     */
    formatRequestData(action: any, params: any): any;
    /**
     * @inner
     */
    formatSignString(params: any): string;
}
import Credential = require("./credential");
import ClientProfile = require("./profile/client_profile");
